# Generated by Django 2.2 on 2019-07-31 18:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0010_merge_20190612_1227'),
        ('tom_targets', '0010_auto_20190715_1657'),
    ]

    operations = [
    ]
