# Generated by Django 2.1.4 on 2019-04-03 16:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0005_auto_20190214_1722'),
    ]

    operations = [
        migrations.AddField(
            model_name='target',
            name='epoch_of_perihelion',
            field=models.FloatField(blank=True, help_text='Julian Years. Max 2100.', null=True, verbose_name='Epoch of Perihelion'),
        ),
        migrations.AddField(
            model_name='target',
            name='scheme',
            field=models.CharField(blank=True, choices=[('MPC_MINOR_PLANET', 'MPC Minor Planet'), ('MPC_COMET', 'MPC Comet'), ('JPL_MAJOR_PLANET', 'JPL Major Planet')], default='', max_length=50, verbose_name='Orbital Element Scheme'),
        ),
    ]
