# Generated by Django 2.1.4 on 2019-01-23 17:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0002_auto_20190117_2248'),
        ('tom_targets', '0002_auto_20190115_2102'),
    ]

    operations = [
    ]
