.thumbnail {
  height: inherit;
  width: 100%;
}

.display {
  margin-left: auto;
  margin-right: auto;
  display: block;
  width: inherit;
  height: auto;
}

.nav-item {
  cursor: pointer;
}

span.featured {
  pointer-events: none;
}

div.observation-form {
  padding-top: 20px;
}
