# Generated by Django 2.2.8 on 2019-12-05 22:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_observations', '0003_auto_20190503_2318'),
    ]

    operations = [
        migrations.CreateModel(
            name='ObservationGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('observation_records', models.ManyToManyField(to='tom_observations.ObservationRecord')),
            ],
        ),
    ]
