<a href="{% url 'tom_observations:list' %}?update_status=True&{{ query_params }}" class="btn btn-primary" title="Update status of all incomplete observations--please be patient!">Update Observations</a>