{% load comments i18n bootstrap4 %}
<form action="{% comment_form_target %}" method="post" class="form">{% csrf_token %}
  {% if next %}
    <div><input type="hidden" name="next" value="{{ next }}"/></div>
  {% endif %}
  {% for field in form %}
    {% if field.is_hidden %}
      <div>{{ field }}</div>
    {% endif %}
  {% endfor %}
  {% bootstrap_field form.comment %}
  <p class="submit">
    <input type="submit" name="post" class="btn btn-primary submit-post" value="{% trans "Post" %}"/>
  </p>
</form>

