{% extends 'tom_common/base.html' %}
{% load bootstrap4 %}
{% block title %}New Target Group{% endblock %}
{% block content %}
<form method="post">{% csrf_token %}
    {{ form.as_p }}
    <input type="submit" value="Create">
</form>
{% endblock %}
