# Generated by Django 2.2.9 on 2020-02-04 23:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_observations', '0006_observationgroup_cadence_strategy'),
    ]

    operations = [
        migrations.CreateModel(
            name='ObservingStrategy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('facility', models.CharField(max_length=50)),
                ('parameters', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]
