# Generated by Django 3.1.5 on 2021-02-04 23:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_alerts', '0003_auto_20190701_2226'),
    ]

    operations = [
        migrations.AlterField(
            model_name='brokerquery',
            name='parameters',
            field=models.JSONField(),
        ),
    ]
