{% extends 'tom_common/base.html' %}
{% load bootstrap4 static %}
{% block title %}Import Targets{% endblock %}
{% block content %}
<h3>Import Targets</h3>
<p>
  Upload a .csv to import targets in bulk. CSV columns must match target attributes.
  View the <a href="{% static 'tom_targets/target_import.csv' %}">example .csv file</a>.
</p>
<form method="POST" action="{% url 'tom_targets:import' %}" enctype="multipart/form-data">
  {% csrf_token %}
  <input type="file" name="target_csv">
  {% buttons %}
  <input type="submit" value="Upload" class="btn btn-primary">
  {% endbuttons %}
</form>
{% endblock %}
