# Generated by Django 3.1.5 on 2021-08-11 00:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0018_auto_20200714_1832'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='target',
            options={},
        ),
    ]
