# Generated by Django 2.1.4 on 2019-05-03 23:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0006_auto_20190403_1659'),
    ]

    operations = [
        migrations.AlterField(
            model_name='target',
            name='epoch_of_perihelion',
            field=models.FloatField(blank=True, help_text='Julian Date.', null=True, verbose_name='Epoch of Perihelion'),
        ),
    ]
