{% extends 'tom_common/base.html' %}
{% block content %}
{% block title %}Confirm delete{% endblock %}
<form class="form" method="post">
  {% csrf_token %}
  <p>Are you sure you want to delete {{ object.name }}?</p>
  <input type="submit" class="btn btn-danger" value="Confirm">
</form>
{% endblock %}
