# Generated by Django 2.1.4 on 2018-12-06 21:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('tom_targets', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ObservationRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('facility', models.CharField(max_length=50)),
                ('parameters', models.TextField()),
                ('observation_id', models.CharField(max_length=2000)),
                ('status', models.CharField(max_length=200)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='tom_targets.Target')),
            ],
            options={
                'ordering': ('-created',),
            },
        ),
    ]
