# Generated by Django 3.1 on 2020-08-18 20:34

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('tom_observations', '0008_observationgroup_cadence_parameters'),
    ]

    operations = [
        migrations.AddField(
            model_name='observationrecord',
            name='user',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to=settings.AUTH_USER_MODEL),
        ),
    ]
