{% extends 'tom_common/base.html' %}
{% load bootstrap4 %}
{% block title %}Confirm delete{% endblock %}
{% block content %}
<form method="post">{% csrf_token %}
    <p>Are you sure you want to delete "{{ object }}"?</p>
    {% buttons %}
    <button type="submit" class="btn btn-danger">Confirm</button>
    {% endbuttons %}
</form>
{% endblock %}
