# Generated by Django 3.1.5 on 2021-02-05 18:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_observations', '0011_auto_20200917_0306'),
    ]

    operations = [
        migrations.AlterField(
            model_name='observationrecord',
            name='parameters',
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name='observationtemplate',
            name='parameters',
            field=models.JSONField(),
        ),
    ]
