# Generated by Django 2.2.2 on 2019-06-26 21:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_alerts', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='brokerquery',
            name='last_run',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name='brokerquery',
            name='modified',
            field=models.DateTimeField(),
        ),
    ]
