# Generated by Django 2.2.2 on 2019-06-26 09:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_dataproducts', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataproduct',
            name='product_id',
            field=models.CharField(help_text='Data product identifier used by the source of the data product.', max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='reduceddatum',
            name='data_type',
            field=models.CharField(choices=[('spectroscopy', 'Spectroscopy'), ('photometry', 'Photometry')], default='', max_length=100),
        ),
    ]
