{% extends 'tom_common/base.html' %}
{% load bootstrap4 crispy_forms_tags %}
{% block title %}Create an Observation Template{% endblock %}
{% block content %}
<h2>Create a new Observation Template for {{ form.facility.value }}</h2>
{% crispy form %}
{% endblock %}