# Generated by Django 2.1.7 on 2019-05-15 09:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0007_auto_20190503_2318'),
        ('tom_targets', '0007_auto_20190423_1455'),
    ]

    operations = [
    ]
