{% extends 'tom_common/base.html' %}
{% load bootstrap4 %}
{% block title %}Login{% endblock %}
{% block content %}
<div class="row justify-content-md-center">
  <div class="col-md-4">
    <form action="{% url 'login' %}?next={{next}}" method="POST" class="form">
      {% csrf_token %}
      {% bootstrap_form form %}
      {% buttons %}
        <button type="submit" class="btn btn-primary">Login</button>
      {% endbuttons %}
    </form>
  </div>
</div>
{% endblock %}
