{% load cache %}
<h3>{{ target.name }}</h3>
{% if target.featured_image %}
{% cache None featured_image target.id %}
<img src="{{ target.featured_image.get_preview }}" id="featured-image" onerror="this.style.display='none'">
{% endcache %}
{% endif %}
