{% extends 'tom_common/base.html' %}
{% load crispy_forms_tags %}
<!-- {% block title %}Manual Observation{% endblock %} -->
{% block content %}
<h3>Confirm Observation Record Creation</h3>
{% crispy form %}
{% endblock %}
