# Generated by Django 3.1 on 2020-09-17 03:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_observations', '0010_manual_create_dynamic_cadence'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ObservingStrategy',
            new_name='ObservationTemplate',
        ),
        migrations.AlterModelOptions(
            name='observationgroup',
            options={'ordering': ('-created', 'name')},
        ),
        migrations.AlterField(
            model_name='dynamiccadence',
            name='active',
            field=models.BooleanField(help_text='Whether or not this DynamicCadence should\n                                           continue to submit observations.', verbose_name='Active'),
        ),
    ]
