# Generated by Django 2.2.2 on 2019-09-12 13:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0011_merge_20190731_1821'),
    ]

    operations = [
        migrations.AddField(
            model_name='target',
            name='perihdist',
            field=models.FloatField(blank=True, help_text='AU', null=True, verbose_name='Perihelion Distance'),
        ),
    ]
