# Generated by Django 2.1.4 on 2019-01-17 22:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='target',
            name='source',
            field=models.TextField(help_text='The original source reference for the target data.', max_length=100, null=True, verbose_name='Target Source'),
        ),
        migrations.AddField(
            model_name='target',
            name='source_location',
            field=models.TextField(help_text='URL or path to original target source reference.', max_length=100, null=True, verbose_name='Target Source Location'),
        ),
    ]
