# Generated by Django 2.1.4 on 2019-01-15 21:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='target',
            name='name2',
            field=models.CharField(default='', help_text='An alternative name for this target', max_length=100, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='target',
            name='name3',
            field=models.CharField(default='', help_text='An alternative name for this target', max_length=100, verbose_name='Name'),
        ),
    ]
