# Generated by Django 2.2.1 on 2019-09-23 18:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0013_manual_20190916_multiple_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='targetlist',
            name='modified',
            field=models.DateTimeField(auto_now=True, help_text='The time which this target was changed in the TOM database.', verbose_name='Last Modified'),
        ),
        migrations.AddField(
            model_name='targetname',
            name='modified',
            field=models.DateTimeField(auto_now=True, help_text='The time which this target was changed in the TOM database.', verbose_name='Last Modified'),
        ),
        migrations.AlterField(
            model_name='target',
            name='name',
            field=models.CharField(default='', help_text="The name of this target e.g. Barnard's star.", max_length=100, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='targetname',
            name='name',
            field=models.CharField(max_length=100, unique=True, verbose_name='Alias for target'),
        ),
    ]
