# Generated by Django 2.2.9 on 2020-02-03 21:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_observations', '0005_auto_20191210_0007'),
    ]

    operations = [
        migrations.AddField(
            model_name='observationgroup',
            name='cadence_strategy',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
    ]
