# Generated by Django 2.2.2 on 2019-07-04 10:10

from django.db import migrations, models
import tom_dataproducts.models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_dataproducts', '0004_auto_20190626_0904'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataproduct',
            name='thumbnail',
            field=models.FileField(default=None, null=True, upload_to=tom_dataproducts.models.data_product_path),
        ),
        migrations.AlterField(
            model_name='dataproduct',
            name='tag',
            field=models.CharField(blank=True, choices=[('photometry', 'Photometry'), ('fits_file', 'Fits File'), ('spectroscopy', 'Spectroscopy'), ('image_file', 'Image File')], default='', max_length=50),
        ),
        migrations.AlterField(
            model_name='reduceddatum',
            name='data_type',
            field=models.CharField(choices=[('spectroscopy', 'Spectroscopy'), ('photometry', 'Photometry'), ('image_file', 'Image File')], default='', max_length=100),
        ),
    ]
