{% extends 'tom_common/base.html' %}
{% load bootstrap4 crispy_forms_tags %}
{% block title %}Query a Broker{% endblock %}
{% block content %}
<h2>{{ form.broker.value }} Query Form</h2>
{% crispy form %}
{% endblock %}
