{% extends 'tom_common/base.html' %}
{% load bootstrap4 %}
{% block title %}Create Data Group {% endblock %}
{% block content %}
<form method="POST">
  {% csrf_token %}
  {% bootstrap_form form %}
  {% buttons %}
  <input type="submit" class="btn btn-primary" value="Save">
  {% endbuttons %}
</form>
{% endblock %}
