# Generated by Django 2.1.7 on 2019-06-12 12:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0009_auto_20190605_1656'),
        ('tom_targets', '0008_merge_20190515_0914'),
    ]

    operations = [
    ]
