body {
  padding-top: 5rem;
}

.content {
  padding: 3rem 1.5rem;
}

.navbar-brand > img {
  max-height: 25px;
}

.input-group-text {
  font-family: monospace;
}

.nav-item {
  cursor: pointer;
}

#login-buttons > li {
  margin: 0 2px 0 2px;
}
