# Generated by Django 3.0.7 on 2020-07-14 18:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0017_auto_20200130_2350'),
    ]

    operations = [
        migrations.AlterField(
            model_name='targetname',
            name='name',
            field=models.CharField(max_length=100, unique=True, verbose_name='Alias'),
        ),
    ]
