# Generated by Django 2.2.2 on 2019-07-15 16:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0009_auto_20190605_1656'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='targetlist',
            options={'ordering': ('-created', 'name')},
        ),
    ]
